/*****
   CHK_DRV()

   A FoxPro API function to check for
   the existence of a hard disk partition
   or a floppy disk in floppy drives.

   Expects 1 or 2 parameters:
      1st parameter must be a drive with colon;
      2nd parameter may or may not be passed,
          if passed and .T. then (in acse of an
          error a message about invalid drive is
          displayed).

   A logical .T. is returned to FoxPro
   if the drive exists and, for floppy drives,
   if a formatted floppy is in the drive and
   the drive is latched.  Otherwise a logical
   .F. is returned without the DOS error message.
   In case of error (if user does not want to
   suppress messages) uses _Execute() to display
   message in WAIT window; message about
   incorrect parameter cannot be suppressed.
*****/

#include <dos.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <io.h>
#include <ctype.h>
#include <errno.h>
#include <pro_ext.h>

int   notReady;
void (__interrupt __far *prev_int)();

void __interrupt __far myErrHandl(union INTPACK r)
{
   if ( (r.w.di & 0x00FF) == 1  ||
        (r.w.di & 0x00FF) == 2  ||
        (r.w.di & 0x00FF) == 12 )
   {
      // ** in case of error
      // ** 01H 'the unit is undefined'
      // ** 02H 'the drive is not ready' (also
      // **     when a network drive does not
      // **     exist any more)
      // ** 0CH 'a general failure occured'
      // **     in case of presence of unformatted
      // **     disk
      // ** just store information about it and ...
      notReady = TRUE;
      // ** ... inform DOS to ignore the error,
      // ** in some cases that means to fail
      // ** the function request
      r.h.al = (unsigned char) 0;
   }
   else
   {
      // ** call original error handler
      _chain_intr(prev_int);
   }
   return;
}

void FAR chk_(ParamBlk FAR *parm)
{
    int     wasError = FALSE;
    char    buf[4];
    char FAR *ptr;
    int     retCode;

    if (parm->p[0].val.ev_length!=2)
       wasError = TRUE;
    else
    {
       ptr = _HandToPtr(parm->p[0].val.ev_handle);
       _MemMove(buf, ptr, 2);
       if (!isalpha(buf[0]) || buf[1]!=':')
          wasError = TRUE;
    }

    if (wasError)
    {
       _Execute("WAIT WINDOW 'Parameter passed to chk_drv must be a letter with colon'");
       _RetLogical(FALSE);
       return;
    }

    buf[2] = '\\';
    buf[3] = '\0';

    // ** floppy must be checked differently:
    // ** to avoid DOS error handling with
    // ** Abort/Retry/Cancel
    notReady = FALSE;

    // ** save interrupt vector address
    // ** for the critical error
    prev_int = _dos_getvect(0x24);

    // ** set vector address for the critical
    // ** error to myErrHandl disabling and
    // ** enabling interrupts for time of doing
    // ** that;
    // ** cannot be done only for a: and b:
    // ** since there may be a call for drive
    // ** on the network which does not exist
    // ** any more
    _disable();
    _dos_setvect(0x24, &myErrHandl);
    _enable();

    retCode = access(buf, F_OK);

    // ** set vector address for the critical
    // ** error back to its original value
    _disable();
    _dos_setvect(0x24, prev_int);
    _enable();
    // ** make sure about success for floppy
    if (notReady)
       retCode = TRUE;

    if (retCode && parm->pCount==2 && parm->p[1].val.ev_length)
       _Execute("WAIT WINDOW 'Invalid drive was specified'");

    _RetLogical(!retCode);
    return;
}

FoxInfo myFoxInfo[] = {
  { "CHK_DRV",
     chk_,
     2,
     "C,.L"}
};

FoxTable _FoxTable = {
    (FoxTable FAR *) 0,
    sizeof( myFoxInfo) / sizeof( FoxInfo ),
    myFoxInfo
};

